
#include <mach/rma1evb_pwic.h>

static DECLARE_MUTEX(pwic_mutex);
static struct i2c_client *pwic_client;

static inline void pwic_reg_select(u32 offset)
{
	i2c_smbus_write_byte_data(pwic_client, PWIC_PMU_ENABLE, offset | 0x8);
}

int pwic_read(u32 offset)
{
	int ret;
	u32 reg;

	down(&pwic_mutex);
	if (offset & 0x10000) {
		pwic_reg_select(offset & 0xff);
		reg = (offset>>8) & 0xff;
	} else
		reg = offset & 0xff;
	ret = i2c_smbus_read_byte_data(pwic_client, reg);
	up(&pwic_mutex);
	return ret;
}
EXPORT_SYMBOL(pwic_read);

int pwic_write(u32 offset, u8 val)
{
	int ret;
	u32 reg;

	down(&pwic_mutex);
	if (offset & 0x10000) {
		pwic_reg_select(offset & 0xff);
		reg = (offset>>8) & 0xff;
	} else
		reg = offset & 0xff;
	ret = i2c_smbus_write_byte_data(pwic_client, reg, val);
	up(&pwic_mutex);
	return ret;
}
EXPORT_SYMBOL(pwic_write);

static int pwic_probe(struct i2c_client *client, const struct i2c_device_id *id)
{
	if (!i2c_check_functionality(client->adapter, I2C_FUNC_SMBUS_BYTE_DATA))
		return -EIO;
	pwic_client = client;
	return 0;
}

static struct i2c_device_id pwic_id[] = {
	{ "pmic-as3608", 0 },
	{ }
};

static struct i2c_driver pwic_driver = {
	.driver = {
		.name = "pwic",
	},
	.probe = pwic_probe,
	.id_table = pwic_id,
};

static int __init pwic_init(void)
{
	return i2c_add_driver(&pwic_driver);
}

device_initcall(pwic_init);
