/*
 * FMSI-AK4648 sound support
 *
 * Copyright (C) 2011  Renesas Electronics Corporation
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/platform_device.h>
#include <linux/i2c.h>
#include <linux/io.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>

#include <../sound/soc/codecs/ak4648.h>

#include "sh_fmsi.h"

static int fmsi_ak4648_dai_init(struct snd_soc_codec *codec)
{
	int ret;

	ret = snd_soc_dai_set_fmt(&ak4648_dai,
#ifdef CONFIG_SND_SOC_SH4_FMSI_MASTER
		/* AK4648 is Slave */
		SND_SOC_DAIFMT_CBS_CFS | SND_SOC_DAIFMT_I2S);
#else
		/* AK4648 is Master */
		SND_SOC_DAIFMT_CBM_CFM | SND_SOC_DAIFMT_I2S);
#endif

	if (ret < 0)
		return ret;

	ret = snd_soc_dai_set_sysclk(&ak4648_dai, 0, 11289600, 0);

	return ret;
}

static struct snd_soc_dai_link fmsi_dai_link = {
	.name		= "ak4648",
	.stream_name	= "ak4648",
	.cpu_dai	= &fmsi_soc_dai,
	.codec_dai	= &ak4648_dai,
	.init		= fmsi_ak4648_dai_init,
	.ops		= NULL,
};

static struct snd_soc_card fmsi_soc_card  = {
	.name		= "FMSI",
	.platform	= &fmsi_soc_platform,
	.dai_link	= &fmsi_dai_link,
	.num_links	= 1,
};

static struct snd_soc_device fmsi_snd_devdata = {
	.card		= &fmsi_soc_card,
	.codec_dev	= &soc_codec_dev_ak4648,
};

static struct platform_device *fmsi_snd_device;

static int __init fmsi_ak4648_init(void)
{
	int ret = -ENOMEM;

	fmsi_snd_device = platform_device_alloc("soc-audio", 1);
	if (!fmsi_snd_device)
		goto out;

	platform_set_drvdata(fmsi_snd_device,
			     &fmsi_snd_devdata);
	fmsi_snd_devdata.dev = &fmsi_snd_device->dev;
	ret = platform_device_add(fmsi_snd_device);

	if (ret)
		platform_device_put(fmsi_snd_device);

out:
	return ret;
}

static void __exit fmsi_ak4648_exit(void)
{
	platform_device_unregister(fmsi_snd_device);
}

module_init(fmsi_ak4648_init);
module_exit(fmsi_ak4648_exit);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Generic SH4 FMSI-AK4648 sound card");
MODULE_AUTHOR("Renesas Electronics");
