/*
 * FSI-WM8978 sound support
 *
 * Copyright (C) 2011 Atmark Techno, Inc.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/platform_device.h>
#include <linux/i2c.h>
#include <linux/io.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>

#include <sound/sh_fsi.h>
#include <../sound/soc/codecs/wm8978.h>

static int fsi_wm8978_dai_init(struct snd_soc_codec *codec)
{
	int ret;

	ret = snd_soc_dai_set_fmt(&wm8978_dai,
		SND_SOC_DAIFMT_CBM_CFM | SND_SOC_DAIFMT_I2S | SND_SOC_DAIFMT_NB_NF);
	if (ret < 0)
		return ret;

	ret = snd_soc_dai_set_sysclk(&wm8978_dai, WM8978_MCLK, 12288000, SND_SOC_CLOCK_IN);

	return ret;
}

static struct snd_soc_dai_link fsi_dai_link = {
	.name		= "wm8978",
	.stream_name	= "wm8978",
	.cpu_dai	= &fsi_soc_dai[0],
	.codec_dai	= &wm8978_dai,
	.init		= fsi_wm8978_dai_init,
	.ops		= NULL,
};

static struct snd_soc_card fsi_soc_card  = {
	.name		= "FSI",
	.platform	= &fsi_soc_platform,
	.dai_link	= &fsi_dai_link,
	.num_links	= 1,
};

static struct snd_soc_device fsi_snd_devdata = {
	.card		= &fsi_soc_card,
	.codec_dev	= &soc_codec_dev_wm8978,
};

static struct platform_device *fsi_snd_device;

static int __init fsi_wm8978_init(void)
{
	int ret = -ENOMEM;

	fsi_snd_device = platform_device_alloc("soc-audio", 0);
	if (!fsi_snd_device)
		goto out;

	platform_set_drvdata(fsi_snd_device,
			     &fsi_snd_devdata);
	fsi_snd_devdata.dev = &fsi_snd_device->dev;
	ret = platform_device_add(fsi_snd_device);

	if (ret)
		platform_device_put(fsi_snd_device);

out:
	return ret;
}

static void __exit fsi_wm8978_exit(void)
{
	platform_device_unregister(fsi_snd_device);
}

module_init(fsi_wm8978_init);
module_exit(fsi_wm8978_exit);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Generic SH4 FSI-WM8978 sound card");
