/*
 *  File Name       : RSPF0_API_S0.h
 *  Function        : SPU OPEN SOUND I/F
 *  Release Version : Ver 1.00
 *  Release Date    : 2011/11/30
 *
 *  Copyright (C) 2011 Renesas Electronics Corporation
 *
 *  This program is free software;you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by Free
 *  Softwere Foundation; either version 2 of License, or (at your option) any
 *  later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program;
 *  If not, write to the Free Software Foundation, Inc., 51 Franklin St,
 *  Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#ifndef __RSPF0_API_S0_H__
#define __RSPF0_API_S0_H__

#include <linux/module.h>

#define RSPF0_PBANK_NUM                 1
#define RSPF0_XBANK_NUM                 1
#define RSPF0_YMEMORY_SIZE              (8192*1)

#define RSPF0_RTN_SUCCESS               0
#define RSPF0_RTN_ERROR_RESOURCE       -1
#define RSPF0_RTN_ERROR_STATE          -2
#define RSPF0_RTN_ERROR_PARAM          -3
#define RSPF0_RTN_ERROR_SETTING        -4
#define RSPF0_RTN_ERROR_DRV            -5
#define RSPF0_RTN_ERROR_SPU_API        -6
#define RSPF0_RTN_ERROR_FATAL          -7

#define RSPF0_FMSI_INFO_UNDERFLOW 0x00000001
#define RSPF0_FMSI_INFO_OVERFLOW  0x00000010
#define RSPF0_FMSI_INFO_FLUSHED   0x00000100

enum {
	RSPF0_SPUAPI_NO_RESPONSE = 0,
	RSPF0_SPUAPI_WAIT_RESPONSE
};

enum {
	RSPF0_SWAP_MODE_NONE = 0x0,
	RSPF0_SWAP_MODE_BYTE,
	RSPF0_SWAP_MODE_WORD,
	RSPF0_SWAP_MODE_BYTE_WORD
};

enum {
	RSPF0_ADDRESS_MODE_29BIT = 0,
	RSPF0_ADDRESS_MODE_32BIT
};

enum {
	RSPF0_SYNC_NO_RESPONSE = 0,
	RSPF0_SYNC_WAIT_RESPONSE
};

enum {
	RSPF0_PORT_CPU = 0,
	RSPF0_PORT_FMSI_A,
	RSPF0_PORT_FMSI_B,
	RSPF0_PORT_MAX
};

enum {
	RSPF0_CHANNEL_MONO   = 1,
	RSPF0_CHANNEL_STEREO = 2
};

enum {
	RSPF0_FIR_NONE  = 0,
	RSPF0_FIR_EXIST = 1
};

enum {
	RSPF0_FIR_OFF = 0,
	RSPF0_FIR_SRC = 2
};

enum {
	RSPF0_FLUSH_OFF = 0,
	RSPF0_FLUSH_ON
};

typedef struct {
	unsigned long buffer_id;
	unsigned long dsp_id;
	unsigned long size;
} RSPF0_sSPUBuffer;

typedef struct {
	unsigned long port_id;
	unsigned long dsp_id;
	unsigned long dma_transfer_mode;
	unsigned long data_unit;
} RSPF0_sSPUDataPort;

typedef struct {
	unsigned long dsp_id;
	unsigned long port_id;
} RSPF0_sSPUFmsiPort;

typedef struct {
	unsigned long dsp_id;
	unsigned long obj_id;
} RSPF0_sSPUSound;

typedef struct RSPF0_Path_ RSPF0_Path;

struct RSPF0_Path_{
	unsigned long dsp_id;
	unsigned long status;
	unsigned long source_port;
	unsigned long dest_port;
	RSPF0_sSPUBuffer buffer[2];
	RSPF0_sSPUDataPort data_port[2];
	RSPF0_sSPUFmsiPort fmsi_port[2];
	RSPF0_sSPUSound sound;
	unsigned long fmsi_port_status[2];
	void (*source_end_callback)(RSPF0_Path *path, unsigned long size);
	void (*dest_end_callback)(RSPF0_Path *path, unsigned long size,
							unsigned long flush);
	void (*fmsi_error_callback)(RSPF0_Path *path, unsigned long info);
};

typedef struct {
	void (*reg_read) (unsigned long *addr, unsigned long *data,
							unsigned long count);
	void (*reg_write) (unsigned long *addr, unsigned long *data,
							unsigned long count);
	long (*wait_for_event) (unsigned long);
	long (*set_event) (void);
	long (*enter_critical_section) (void);
	long (*leave_critical_section) (void);
} RSPF0_Func;

typedef struct {
	unsigned long swap_mode;
	unsigned long source_port;
	unsigned long dest_port;
	void (*source_end_callback)(RSPF0_Path *path, unsigned long size);
	void (*dest_end_callback)(RSPF0_Path *path, unsigned long size,
							unsigned long flush);
	void (*fmsi_error_callback)(RSPF0_Path *path, unsigned long info);

	unsigned long channel;

	struct {
		unsigned long fir_mode;
	} sound;
} RSPF0_OpenParam;



#ifdef __cplusplus
extern "C" {
#endif
extern long RSPF0_Init(unsigned long dsp_id, RSPF0_Func *func,
						unsigned long address_mode);
extern long RSPF0_Quit(unsigned long dsp_id);
extern void RSPF0_Interrupt(unsigned long dsp_id);
extern long RSPF0_Open(unsigned long dsp_id, RSPF0_Path *path,
							RSPF0_OpenParam *param);
extern long RSPF0_Close(RSPF0_Path *path);
extern long RSPF0_FmsiStart(RSPF0_Path *path, unsigned long sync);
extern long RSPF0_FmsiStop(RSPF0_Path *path, unsigned long flush,
							unsigned long sync);
extern long RSPF0_SourceTransferStart(RSPF0_Path *path, short *source,
							unsigned long size);
extern long RSPF0_DestTransferStart(RSPF0_Path *path, short *dest,
							unsigned long size);
extern long RSPF0_SetSrc(RSPF0_Path *path, unsigned long ratio,
							unsigned long sync);
extern unsigned long RSPF0_GetVersion(void);
extern long RSPF0_IoWrite(unsigned long dsp_id, unsigned long addr,
				unsigned long reg_value, unsigned long sync);
extern long RSPF0_IoRead(unsigned long dsp_id, unsigned long addr,
						unsigned long *reg_value);
extern long RSPF0_Clear(RSPF0_Path *path);
#ifdef __cplusplus
}
#endif
#endif /* __RSPF0_API_S0_H__ */

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("SPU OPEN SOUND I/F Ver 1.0");
MODULE_AUTHOR("Renesas Electronics");
