/*
 *  File Name       : RSPF0_spu_api_dataport.c
 *  Function        : SPU OPEN SOUND I/F
 *  Release Version : Ver 1.00
 *  Release Date    : 2011/11/30
 *
 *  Copyright (C) 2011 Renesas Electronics Corporation
 *
 *  This program is free software;you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by Free
 *  Softwere Foundation; either version 2 of License, or (at your option) any
 *  later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program;
 *  If not, write to the Free Software Foundation, Inc., 51 Franklin St,
 *  Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#include "RSPF0_spu_api_common.h"
#include "RSPF0_spu_api_dataport.h"

#define DATAPORT_MAX_NUM 16

#define CLASS_ID 0x10

/* sign extention macro */
#define INT24_TO_INT(x)             \
	(((x) & 0x00800000) ?       \
	((x) | 0xff000000) :  (x))

enum {
	API_DATAPORT_CREATE_REQUEST     = SPUAPI_API_ID(CLASS_ID, 0x00),
	API_DATAPORT_CREATE_RESPONSE    = SPUAPI_API_ID(CLASS_ID, 0x80),
	API_DATAPORT_DELETE_REQUEST     = SPUAPI_API_ID(CLASS_ID, 0x01),
	API_DATAPORT_DELETE_RESPONSE    = SPUAPI_API_ID(CLASS_ID, 0x81),
	API_DATAPORT_SET_PARAM_REQUEST  = SPUAPI_API_ID(CLASS_ID, 0x02),
	API_DATAPORT_SET_PARAM_RESPONSE = SPUAPI_API_ID(CLASS_ID, 0x82),
	API_DATAPORT_GET_PARAM_REQUEST  = SPUAPI_API_ID(CLASS_ID, 0x03),
	API_DATAPORT_GET_PARAM_RESPONSE = SPUAPI_API_ID(CLASS_ID, 0x83),
	API_DATAPORT_ENABLE_REQUEST     = SPUAPI_API_ID(CLASS_ID, 0x04),
	API_DATAPORT_ENABLE_RESPONSE    = SPUAPI_API_ID(CLASS_ID, 0x84),
	API_DATAPORT_DISABLE_REQUEST    = SPUAPI_API_ID(CLASS_ID, 0x05),
	API_DATAPORT_DISABLE_RESPONSE   = SPUAPI_API_ID(CLASS_ID, 0x85),
	API_DATAPORT_RESET_REQUEST      = SPUAPI_API_ID(CLASS_ID, 0x06),
	API_DATAPORT_RESET_RESPONSE     = SPUAPI_API_ID(CLASS_ID, 0x86),
	API_DATAPORT_GET_TRANSFER_COUNT_REQUEST  =
						SPUAPI_API_ID(CLASS_ID, 0x07),
	API_DATAPORT_GET_TRANSFER_COUNT_RESPONSE =
						SPUAPI_API_ID(CLASS_ID, 0x87),
	API_DATAPORT_TRANSFER_START     = SPUAPI_API_ID(CLASS_ID, 0x08),
	API_DATAPORT_TRANSFER_REQUEST   = SPUAPI_API_ID(CLASS_ID, 0x88),
	API_DATAPORT_TRANSFER_END       = SPUAPI_API_ID(CLASS_ID, 0x89),
};
typedef struct {
	sSPUDataPort *port;
/*	tSPUDataPortTransferRequestCallBack request_callback;*/
	tSPUDataPortTransferEndCallBack end_callback;
} sDataPortHandle;

#define DATAPORT_PROCESSED_SIZE_PARAM_MASK  0x00FFFFFF

static sDataPortHandle handle[SPUAPI_DSP_MAX][DATAPORT_MAX_NUM];
static unsigned long response[SPUAPI_DSP_MAX][SPUAPI_MESSAGE_LENGTH];

static void SPUDataPort_ApiReceiveHandler(unsigned long dsp_id,
	unsigned long api_id, unsigned long *data, unsigned long size);

/* General API */
long SPUDataPort_Init(unsigned long dsp_id)
{
	unsigned long i;

	if (SPUAPI_RegisterApiReceiveHandler
	(dsp_id, CLASS_ID, SPUDataPort_ApiReceiveHandler) != 0)
		return -1;

	for (i = 0; i < DATAPORT_MAX_NUM; i++)
		handle[dsp_id][i].port = 0;

	return 0;
}

long SPUDataPort_SetCallBack(sSPUDataPort *port,
				tSPUDataPortTransferEndCallBack end_cb) {
	unsigned long i;

	if (!port)
		return SPUAPI_ERROR_PARAM;
	for (i = 0; i < DATAPORT_MAX_NUM; i++) {
		if (handle[port->dsp_id][i].port) {
			if (handle[port->dsp_id][i].port->port_id ==
				port->port_id)
				break;
		}
	}
	if (i == DATAPORT_MAX_NUM)
		return SPUAPI_ERROR_NO_OBJECT;
/*	handle[port->dsp_id][i].request_callback = request_cb;*/
	handle[port->dsp_id][i].end_callback = end_cb;
	return 0;
}

long SPUDataPort_Create(unsigned long dsp_id, sSPUDataPort *port)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	long result;
	unsigned long i;

	if (!port || dsp_id >= SPUAPI_DSP_MAX)
		return SPUAPI_ERROR_PARAM;

	for (i = 0; i < DATAPORT_MAX_NUM; i++) {
		if (!handle[dsp_id][i].port)
			break;
	}
	if (i == DATAPORT_MAX_NUM)
		return SPUAPI_ERROR_RESOURCE;

	api_id = API_DATAPORT_CREATE_REQUEST;
	size = 0;
	response_api = API_DATAPORT_CREATE_RESPONSE;

	if (SPUAPI_SendApi(dsp_id, api_id, 0, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	result = (long)response[dsp_id][0];
	if (result == 0) {
		port->port_id = response[dsp_id][1];
		port->dsp_id = dsp_id;
		handle[dsp_id][i].port = port;
	}

	return result;
}

long SPUDataPort_Delete(sSPUDataPort *port)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	long result;
	unsigned long data[1];
	unsigned long i;

	if (!port)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_DATAPORT_DELETE_REQUEST;
	size         = 1;
	data[0]      = port->port_id;
	response_api = API_DATAPORT_DELETE_RESPONSE;

	if (SPUAPI_SendApi(port->dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	result = (long)response[port->dsp_id][0];
	if (result == 0) {
		port->port_id = 0;
		for (i = 0; i < DATAPORT_MAX_NUM; i++) {
			if (handle[port->dsp_id][i].port == port) {
				handle[port->dsp_id][i].port = 0;
				break;
			}
		}
		if (i == DATAPORT_MAX_NUM)
			return SPUAPI_ERROR_NO_OBJECT;
	}

	return result;
}

long SPUDataPort_SetParam(sSPUDataPort *port, sSPUDataPortParam *param,
				sSPUBuffer *buffer, unsigned long sync) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[4];

	if (!port || !param)
		return SPUAPI_ERROR_PARAM;

	if (param->data_unit > SPUDATAPORT_DATAUNIT_HALFWORD ||
	param->direction > SPUDATAPORT_DIRECTION_OUT ||
	param->dma_ch > 1 ||
	param->dma_transfer_mode > 7 ||
	param->dma_swap_mode > 3 ||
	param->end_notify_condition > SPUDATAPORT_END_CONDITION_BUFFER_AVAILABLE
	)
		return SPUAPI_ERROR_PARAM;
	if ((param->data_unit == SPUDATAPORT_DATAUNIT_HALFWORD &&
	param->dma_transfer_mode !=
	SPUDATAPORT_TRANSFER_MODE_BURST_ALIGNMENT3) ||
	(param->data_unit == SPUDATAPORT_DATAUNIT_BYTE &&
	param->dma_transfer_mode != SPUDATAPORT_TRANSFER_MODE_BURST_SEAMLESS) ||
	(param->data_unit == SPUDATAPORT_DATAUNIT_WORD &&
	param->dma_transfer_mode > SPUDATAPORT_TRANSFER_MODE_BURST_ALIGNMENT2))
		return SPUAPI_ERROR_PARAM;
	if ((param->end_notify_condition ==
	SPUDATAPORT_END_CONDITION_BUFFER_AVAILABLE) &&
	(param->data_unit != SPUDATAPORT_DATAUNIT_BYTE ||
	param->direction != SPUDATAPORT_DIRECTION_IN))
		return SPUAPI_ERROR_PARAM;

	api_id = API_DATAPORT_SET_PARAM_REQUEST;

	data[0] = port->port_id;
	data[1] = buffer->buffer_id;
	data[2] = (param->direction << 20) |
		(param->data_unit << 16) |
		(param->dma_ch << 8) |
		(param->dma_transfer_mode << 4) |
		(param->dma_swap_mode);
	data[3] = param->end_notify_condition << 16;

	size = 4;

	if (sync)
		response_api = API_DATAPORT_SET_PARAM_RESPONSE;
	else{
		response_api = 0;	/* The response doesn't meet. */
		api_id = SPUAPI_NO_RESPONSE_API(api_id);
						/* Restrain the response. */
	}

	if (SPUAPI_SendApi(port->dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	port->dma_transfer_mode = param->dma_transfer_mode;
	port->data_unit = param->data_unit;

	if (sync)
		return response[port->dsp_id][0];
	else
		return 0;
}

long SPUDataPort_Enable(sSPUDataPort *port, unsigned long sync)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (!port)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_DATAPORT_ENABLE_REQUEST;
	size         = 1;
	data[0]      = port->port_id;

	if (sync)
		response_api = API_DATAPORT_ENABLE_RESPONSE;
	else{
		response_api = 0;	/* The response doesn't meet. */
		api_id = SPUAPI_NO_RESPONSE_API(api_id);
						/* Restrain the response. */
	}

	if (SPUAPI_SendApi(port->dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	if (sync)
		return response[port->dsp_id][0];
	else
		return 0;
}

long SPUDataPort_Disable(sSPUDataPort *port, unsigned long *rest_size,
							unsigned long sync) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (!port)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_DATAPORT_DISABLE_REQUEST;
	size         = 1;
	data[0]      = port->port_id;

	if (sync)
		response_api = API_DATAPORT_DISABLE_RESPONSE;
	else{
		response_api = 0;	/* The response doesn't meet. */
		api_id = SPUAPI_NO_RESPONSE_API(api_id);
						/* Restrain the response. */
	}

	if (SPUAPI_SendApi(port->dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	if (sync) {
		if (rest_size)
			*rest_size = response[port->dsp_id][2];
		return response[port->dsp_id][0];
	} else
		return 0;
}


long SPUDataPort_TransferStart(sSPUDataPort *port, unsigned long addr,
							unsigned long tr_size) {
	unsigned long api_id;
	unsigned long size;
	unsigned long data[4];
	unsigned long dma_addr;
	unsigned long offset;
	unsigned long unit_size[] = {4, 1, 2};

	if (!port)
		return SPUAPI_ERROR_PARAM;

	offset = unit_size[port->data_unit] * tr_size;

	if (SPUAPI_CheckDmaParam(addr, offset, port->dma_transfer_mode) != 0)
		return SPUAPI_ERROR_PARAM;

	dma_addr     = SPUAPI_DmaAddress(addr);

	api_id       = API_DATAPORT_TRANSFER_START;
	data[0]      = port->port_id;
	data[1]      = dma_addr>>8;
	data[2]      = dma_addr;
	data[3]      = tr_size;
	size         = 4;

	if (SPUAPI_SendApi(port->dsp_id, api_id, data, size, 0) < 0)
		return SPUAPI_ERROR_DRIVER;

	return 0;
}

long SPUDataPort_GetTransferCount(sSPUDataPort *port, unsigned long *rest_size)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];
	long result;

	if (!port)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_DATAPORT_GET_TRANSFER_COUNT_REQUEST;
	size         = 1;
	data[0]      = port->port_id;
	response_api = API_DATAPORT_GET_TRANSFER_COUNT_RESPONSE;

	if (SPUAPI_SendApi(port->dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	result      = (long)response[port->dsp_id][0];
	if (rest_size)
		*rest_size  = DATAPORT_PROCESSED_SIZE_PARAM_MASK &
						response[port->dsp_id][2];
	return result;
}

static sDataPortHandle *get_handle_from_port_id(unsigned long dsp_id,
							unsigned long port_id) {
	unsigned long i;

	for (i = 0; i < DATAPORT_MAX_NUM; i++) {
		if (handle[dsp_id][i].port && handle[dsp_id][i].port->port_id ==
			port_id)
			return &handle[dsp_id][i];
	}
	return 0;
}

static void SPUDataPort_ApiReceiveHandler(unsigned long dsp_id,
		unsigned long api_id, unsigned long *data, unsigned long size) {
	unsigned long i;
	sDataPortHandle *handle;

	switch (api_id) {
#if 0
	case API_DATAPORT_TRANSFER_REQUEST:
		handle = get_handle_from_port_id(dsp_id, data[0]);
		if (handle && handle->request_callback)
			handle->request_callback(handle->port, data[1]
								/* size */);
		break;
#endif
	case API_DATAPORT_TRANSFER_END:
		{
			int flush = 0;
			flush = data[2];
			handle = get_handle_from_port_id(dsp_id, data[0]);
			if (handle && handle->end_callback)
				handle->end_callback(handle->port, data[1],
									flush);
		}
		break;
	default:
		for (i = 0; i < size; i++)
			response[dsp_id][i] = INT24_TO_INT(data[i]);
								/* pgr0788 */
		break;
	}
}

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("SPU OPEN SOUND I/F Ver 1.0");
MODULE_AUTHOR("Renesas Electronics");
